@@ -19,4 +19,20 @@
 #  works or verbatim, obfuscated, compiled or rewritten versions of any
 #  part of this work is illegal and unethical regarding the effort and
 #  time spent here.
-GLOBAL = {"CACHE": None}  # TODO replace by disk()
+from pathlib import Path
+from typing import TypeVar, Dict
+
+VT = TypeVar("VT")
+
+GLOBAL = {
+    "cache": f"{Path.home()}/{'.ldict/shelve.db'}",
+    "history": True
+}
+
+
+def setcache(cache: Dict[str, VT]):
+    GLOBAL["cache"] = cache
+
+
+def sethistory(state: bool):
+    GLOBAL["history"] = state
