@@ -33,13 +33,13 @@ from orjson import dumps, OPT_SORT_KEYS
 from garoupa import ø40
 from ldict.appearance import ldict2txt, decolorize, ldict2dic
 from ldict.apply import delete, application
+from ldict.config import GLOBAL
 from ldict.customjson import CustomJSONEncoder
 from ldict.data import key2id
-from ldict.exception import ReadOnlyLdict, WrongKeyType, WrongValueType, OverwriteException
+from ldict.exception import WrongKeyType, WrongValueType, OverwriteException, check
 from ldict.functionspace import FunctionSpace
 from ldict.history import extend_history, rewrite_history
 from ldict.lazy import islazy
-from ldict.persistence import GLOBAL
 from ldict.persistence.cached import cached
 
 VT = TypeVar("VT")
@@ -138,15 +138,11 @@ class Ldict(UserDict, Dict[str, VT]):
     """
 
     def __init__(self, /, _dictionary=None, identity=ø40, readonly=False, **kwargs):
-        self.identity, self.readonly, self.digits, self.version = identity, readonly, identity.digits, identity.version
-        self.blobs = {}
-        self.hashes = {}
-        self.hoshes = {}
-        self.hosh = identity
-        self.rho = identity.rho
-        self.delete = identity.delete
-        self.history = {}
-        self.last = None
+        self.hosh = self.identity = identity
+        self.readonly, self.digits, self.version = readonly, identity.digits, identity.version
+        self.rho, self.delete = identity.rho, identity.delete
+        self.blobs, self.hashes, self.hoshes = {}, {}, {}
+        self.history, self.last = {}, None
         self.rnd = Random()
         super().__init__()
         self.data.update(id=identity.id, ids={})
@@ -154,19 +150,10 @@ class Ldict(UserDict, Dict[str, VT]):
         self.__name__ = self.id[:10]
 
     def __setitem__(self, key: str, value):
-        if self.readonly:
-            raise ReadOnlyLdict(f"Cannot change a readonly ldict ({self.id}).", key)
-        if not isinstance(key, str):
-            raise WrongKeyType(f"Key must be string, not {type(key)}.", key)
-        if callable(value):
-            raise WrongValueType(f"A value for the field [{key}] cannot have type {type(value)}. "
-                                 f"For (pseudo)inplace function application, use operator >>= instead")
-        if isinstance(value, Ldict):
-            value = value.clone(readonly=True)
-
+        check(self.id, self.readonly, key, value)
+        value = value.clone(readonly=True) if isinstance(value, Ldict) else value
         if key in self.data:
             del self[key]
-
         if hasattr(value, "hosh"):
             if isinstance(value, Ldict):
                 self.hashes[key] = value.hosh
@@ -182,7 +169,7 @@ class Ldict(UserDict, Dict[str, VT]):
         self.data[key] = value
         self.data["id"] = self.hosh.id
         self.data["ids"][key] = self.hoshes[key].id
-        extend_history(self, self.hoshes[key])
+        self.last = extend_history(self.history, self.last, self.hoshes[key])
 
     def __getitem__(self, item):
         if not isinstance(item, str):
@@ -195,10 +182,7 @@ class Ldict(UserDict, Dict[str, VT]):
         return self.data[item]
 
     def __delitem__(self, key):
-        if self.readonly:
-            raise ReadOnlyLdict(f"Cannot change a readonly ldict ({self.id}).", key)
-        if not isinstance(key, str):
-            raise WrongKeyType(f"Key must be string, not {type(key)}.", key)
+        check(self.id, self.readonly, key)
         if key in self.blobs:
             del self.blobs[key]
             del self.hashes[key]
@@ -209,7 +193,7 @@ class Ldict(UserDict, Dict[str, VT]):
         del self.data[key]
         self.data["id"] = self.hosh.id
         del self.data["ids"][key]
-        rewrite_history(self, deleted)
+        self.last = rewrite_history(self.history, self.last, deleted, self.hoshes)
 
     def __repr__(self, all=False):
         return ldict2txt(self, all)
@@ -387,9 +371,7 @@ class Ldict(UserDict, Dict[str, VT]):
         # from ldict import Empty
         # if isinstance(other, FunctionSpace) and isinstance(other[0], Empty):
         #     raise EmptyNextToGlobalCache("Cannot use ø after ^ due to Python precedence rules.")
-        c = cached(self, GLOBAL["CACHE"])
-        c = cached(self, GLOBAL["CACHE"]) >> other
-        return cached(self, GLOBAL["CACHE"]) >> other
+        return cached(self, GLOBAL["cache"]) >> other
 
     @property
     def idc(self):
